<template>
  <div :id="'UI-dynamic-'+areasName+componentVO.oid" class="UI-dynamic">
    <avue-crud ref="dataTable"
               v-model="form"
               :data="tableList"
               :option="option"
               :page.sync="pageType"
               :table-loading="loading"
               @on-load="onLoad"
               @search-reset="searchReset"
               @refresh-change="handleRefresh"
               @current-change="currentChange"
               @size-change="sizeChange"
               @row-click="rowClickChange"
               @search-change='searchChange'
               @selection-change="selectChange">
      <!--top区域按钮-->
      <template slot="menuLeft" slot-scope="scope">
        <dynamic-button :componentVO="componentVO"
                        :key="areasName+'buttons-'+componentVO.oid"
                        :butttonList="componentVO.buttons" :dataStore="selectList" LocationType="top" :sourceData="sourceData"
                        type="table"></dynamic-button>
      </template>

      <!--menu区域按钮-->
      <template slot="menu" slot-scope="scope">
        <dynamic-button :componentVO="componentVO" :butttonList="componentVO.buttons" :scope="scope" :dataStore="selectList" :sourceData="sourceData" LocationType="menu"
                        type="table"></dynamic-button>
      </template>
    </avue-crud>
  </div>
</template>

<script>
import {validatenull} from '@/util/validate'
import {getList} from '@/api/base/ui'

export default {
  name: "dynamic-table",
  props: {
    //ui上下文的业务类型（或链接类型）
    uiBtmType: {
      type: String
    },
    //ui上下文
    uiContext:{
      type: String
    },
    componentVO: {
      type: Object,
      default: {}
    },
    inDialog: {
      type: Boolean,
      default: false
    },
    canEdit:{
      //内容是否可编辑
      type:Boolean,
      default:false
    },
    areasName: {
      type: String,
      default: ''//westArea导航区
    },
    sourceData: {
      //菜单源数据或者弹窗时按钮所属区域的上一区域选中数据
      type: Object,
      default: {}
    },
    //上一区域业务类型
    sourceBtmType:{
      type: String
    },
    paramVOS: {
      type: Object,
      default: {}
    },
    isShow: {
      //所在区域是否已显示，针对tab和collapse
      type: Boolean,
      default: true
    },
    dataStore: {
      //弹窗时按钮所属区域选中数据
      type: Array,
      default: []
    },
    tableType: {
      type: String
    }
  },
  data() {
    return {
      currentDefineVO:this.componentVO.uiComponentType=='table'?this.componentVO.tableDefineVO:this.componentVO.treeTableDefineVO,
      parentHeight: '100%',//当前组件根节点元素高度
      form: {},
      query:{},
      params:{},
      loading: false,
      page: {
        pageSize: 10,
        currentPage: 1,
        total: 0,
      },
      simplePage: {
        currentPage: 1,
        total: 0,
        pagerCount: 4,
        layout: "prev, pager, next"
      },
      //表格数据
      tableList: [],
      option: {
        index: true,
        addBtn: false,
        editBtn: false,
        delBtn: false,
        selection: true,
        tip: false,
        height: '100%',
        calcHeight: 15,
        indexFixed: false,
        menuFixed: this.areasName === 'westArea' ? false : 'right',
        searchMenuSpan:12,
        searchShow:false,
        column: [],
      },
      selectList: [],
      columnType: {
        text: "input",
        combox: "select",
        truefalse: "switch",
        number: "number",
        textarea: "textarea",
        datetime: "datetime",
        date: "date",
        refer: "refer",
      },
      sourceDataMapParams:{}
    }
  },
  computed: {
    pageType() {
      return this.areasName === 'westArea' ? this.simplePage : this.page;
    }
  },
  watch: {
    parentHeight: {
      handler(newval) {
        if (newval > 50) {
          this.option.height = newval;
          //doLayout不生效，手动设置表格高度
          this.$children[0].$children[1].$children[1].$children[0].$el.style.height = newval + 'px';
          /*this.$nextTick(() => {
            if (this.$refs.dataTable && this.$refs.dataTable.doLayout) {
              this.$refs.dataTable.doLayout();
            }
          })*/
        }
      }
    },
    isShow: {
      handler(newval) {
        if (newval && this.$el.clientHeight > 50) {
          this.parentHeight = this.$el.clientHeight - this.$children[0].$children[1].$children[0].$el.clientHeight - this.$children[0].$children[2].$el.clientHeight - 5;
        }
        if(newval){
          this.handleRefresh();
        }
      }
    },
    'sourceData':{
      handler(newval) {
        if(newval) {
          this.sourceDataMapParams=this.sourceDataMap();
          this.getParams();
          this.handleRefresh();
        }
      }
    }
  },
  created() {
    this.getParams();
    this.option.column = this.updatedColumns();
    if (this.componentVO.uiComponentType == 'TreeTable') {
      //树表
      this.option.rowKey = this.currentDefineVO.treeCurrentField || 'oid';
      this.option.rowParentKey = this.currentDefineVO.treeParentField || 'parentOid'
    }
    this.page = {
      pageSize: this.currentDefineVO.pageVO ? this.currentDefineVO.pageVO.limit : 10,
      currentPage: this.currentDefineVO.pageVO ? this.currentDefineVO.pageVO.page : 1,
      total: 0,
      pageSizes: this.currentDefineVO.limits || [10, 20, 30, 40, 50, 100]
    };
  },
  mounted() {
    if (this.$el.clientHeight > 50) {
      //父元素高度-按钮高度-分页高度
      this.parentHeight = this.$el.clientHeight - this.$children[0].$children[1].$children[0].$el.clientHeight - this.$children[0].$children[2].$el.clientHeight - 5;
    }
  },
  methods: {
    updatedColumns: function () {
      const queryFields = !validatenull(this.currentDefineVO.queryColumns) ? this.currentDefineVO.queryColumns.map(item => item.field) : [];
      return this.currentDefineVO.cols[0].map(item => {
        const search = queryFields.includes(item.field); // 判断 field 是否在 queryColumns 里
        if (this.currentDefineVO.btmType == 'fileobject' && item.field == 'name') {
          //是文件
          item.formatter = function (d) {
            return '<a class="layui-btn layui-btn-intable"  lay-event="PREVIEW">' + d.name + '</a>'
          }
        }
        let formatter = item.formatter || item.templet;
        if (typeof formatter == "string" && formatter != '') {
          formatter = eval("(" + formatter + ")");
        }
        item.type = this.columnType[item.fieldType] || item.fieldType;
        return {
          ...item,
          prop: item.field,
          label: item.title,
          search: search,
          formatter: formatter
        };
      });
    },
    sourceDataMap: function () {
      const sourceDataMap = {};
      if (Object.keys(this.sourceData).length>0) {
        if(this.sourceData.oid ) {
          if (this.sourceData.oid.indexOf('@vcitreesep@') > -1) {
            this.sourceData.oid = this.sourceData.oid.split('@vcitreesep@')[1];
          }
          sourceDataMap.sourceBtmName = this.sourceBtmType;;
          sourceDataMap.sourceOid = this.sourceData.oid;
        }
        for (let i in this.sourceData) {
          let item = this.sourceData[i]
          if (item && item.constructor === Object) return;
          if (i == 'type' || i == 'context' || i == 'content') return;
          sourceDataMap['sourceData["' + i + '"]'] = item
        }
      }

      if (Object.keys(this.paramVOS).length>0) {
        for (let i in this.paramVOS) {
          let item = this.paramVOS[i]
          if (item && item.constructor === Object) return;
          if (i == 'type' || i == 'context' || i == 'content') return;
          sourceDataMap['sourceData["' + i + '"]'] = item
        }
      }
      return sourceDataMap;
    },
    getParams: function () {
      const tableParams = {
        btmname: this.currentDefineVO.btmType,
        btmType:this.currentDefineVO.btmType,
        tableDefineId: this.currentDefineVO.id,
        componentOid: this.componentVO.oid,
        uiDefineId: this.uiContext,
        linkTypeFlag: this.currentDefineVO.linkTypeFlag,
        treeTableFlag:this.componentVO.uiComponentType == 'TreeTable'
      };

      const sourceDataMapList = this.sourceDataMapParams;

      this.params = Object.assign({},tableParams, sourceDataMapList);
    },
    onLoad(page, params = {}) {
      if (Object.keys(this.sourceData).length>0 && this.isShow) {
        this.loading = true;
        getList(page.currentPage, page.pageSize, Object.assign(params,this.params,this.query)).then(res => {
          let data = [];
          if (res.data && res.data.data) {
            data = res.data.data;
            this.page.total = res.data.total;
          } else {
            data = res.data;
            this.page.total = res.total;
          }
          if (!data || data == null) {
            data = [];
          }
          this.tableList = data;
          this.loading = false;
          this.selectionClear();
        }).catch(error => {
          this.$message.error(error);
          this.loading = false;
        });
      }
    },
    rowClickChange(row){
      this.$refs.dataTable.toggleRowSelection(row);
    },
    selectChange(row) {
      this.selectList = row;
      this.$emit("setDataStore", {
        area: this.areasName,
        type:this.componentVO.uiComponentType,
        btmType:this.currentDefineVO.btmType,
        dataStore:row
      });
    },
    currentChange(currentPage) {
      this.page.currentPage = currentPage;
    },
    sizeChange(pageSize) {
      this.page.pageSize = pageSize;
    },
    searchChange(params,done){
      this.query = params;
      this.page.currentPage = 1;
      this.onLoad(this.page);
      done();
    },
    searchReset() {
      this.query = {};
      this.onLoad(this.page);
    },
    selectionClear() {
      this.selectionList = [];
      this.$refs.dataTable.toggleSelection();
    },
    handleRefresh() {
      this.onLoad(this.page, this.query);
    }
  }
}
</script>

<style scoped>

</style>
